﻿
namespace NiknakV2.Business.Gallery
{
    public class Album : FileSystemInfo, IGalleryEntity
    {

        private byte[] _ImageStack;
        private bool _ImageStackInError;
        
        public bool ImageStackInError
        {
            get { return _ImageStackInError; }
        }

        public byte[] ImageStack
        {
            get
            {
                if (_ImageStack == null || _ImageStack.Length.Equals(0) || ImageStackInError)
                    GetImageStack();
                return _ImageStack;
            }
        }

        [STAThread]
        private byte[] GetImageStack()
        {
            try
            {
                Thread imageStackworker = new Thread(new ThreadStart(BuildImageStack));
                imageStackworker.SetApartmentState(ApartmentState.STA);
                imageStackworker.Name = "GetImageStack";
                imageStackworker.Start();
                imageStackworker.Join();
            }
            catch (Exception ex)
            {
                return NiknakV2.Business.Gallery.Drawing.ImageError.RenderAsByteArray(ex.Message);
            }

            return _ImageStack;
        }

        private void BuildImageStack()
        {
            try
            {
                NiknakV2.Business.Gallery.Drawing.ImageStack myStack = new NiknakV2.Business.Gallery.Drawing.ImageStack(this.FullName);

                //setup options
                int maxDimension = ImageStackMaxDimension;
                if (maxDimension.Equals(0))
                    maxDimension = 92;

                myStack.FolderImagePath = FolderImagePath;
                myStack.MaxDimension = maxDimension;

                //specify which pictures to use
                List<Picture> landscapePics = LandscapePictures();

                if (_XmlImageStackPic1Index.HasValue && _XmlImageStackPic1Index < Pictures.Count)
                    myStack.Picture1 = Pictures[_XmlImageStackPic1Index.Value];
                else if (landscapePics.Count > 0)
                    myStack.Picture1 = landscapePics[0];

                if (_XmlImageStackPic2Index.HasValue && _XmlImageStackPic2Index < Pictures.Count)
                    myStack.Picture2 = Pictures[_XmlImageStackPic2Index.Value];
                else if (landscapePics.Count > 1)
                    myStack.Picture2 = landscapePics[1];

                if (_XmlImageStackPic3Index.HasValue && _XmlImageStackPic3Index < Pictures.Count)
                    myStack.Picture3 = Pictures[_XmlImageStackPic3Index.Value];
                else if (landscapePics.Count > 2)
                    myStack.Picture3 = landscapePics[2];

                //work some magic
                _ImageStack = myStack.Render();
                _ImageStackInError = myStack.InError;
            }
            catch (Exception ex)
            {
                _ImageStackInError = true;
                _ImageStack = NiknakV2.Business.Gallery.Drawing.ImageError.RenderAsByteArray(ex.Message);
            }
        }

    }
}
