﻿
namespace NiknakV2.Presentation.Gallery.Drawing
{
    class ImageStack : IHttpHandler
    {
        private byte[] StackBuffer { get; set; }
        private HttpContext Context { get; set; }

        private int MaxDimension { get; set; }
        private System.Drawing.Size TargetSize { get; set; }

        public void ProcessRequest(System.Web.HttpContext context)
        {
            Context = context;

            try
            {
                string qString = context.Request.ServerVariables["QUERY_STRING"];

                //MaxDimension currently only used for width hence param name
                int maxWidth;
                if (int.TryParse(context.Request.QueryString["maxwidth"], out maxWidth))
                    MaxDimension = maxWidth;

                int sizeX, sizeY = 0;
                if (int.TryParse(context.Request.QueryString["sizex"], out sizeX) && int.TryParse(context.Request.QueryString["sizey"], out sizeY))
                    TargetSize = new System.Drawing.Size(sizeX, sizeY);

                NiknakV2.Business.Gallery.Album myAlbum = new NiknakV2.Business.Gallery.Album(pathToAlbum);
                myAlbum.FolderImagePath = context.Server.MapPath("~" + NiknakV2.Business.Gallery.Config.General.FolderImagePath);
                StackBuffer = myAlbum.ImageStack;

                context.Response.Clear();
                context.Response.ContentType = @"image/jpeg";

                if (sizeX.Equals(0) || sizeY.Equals(0))
                {
                    //no scaling required
                    context.Response.BufferOutput = true;
                    context.Response.BinaryWrite(StackBuffer);
                }
                else
                {
                    //scale using gdi for decent quality
                    using (MemoryStream imageData = new MemoryStream(StackBuffer))
                    {
                        using (System.Drawing.Image mySourceImage = System.Drawing.Image.FromStream(imageData))
                        {
                            if (mySourceImage.Width != sizeX || mySourceImage.Height != sizeY)
                            {
                                using (Bitmap myDestinationBitmap = new Bitmap(sizeX, sizeY))
                                {
                                    using (Graphics myGraphic = Graphics.FromImage(myDestinationBitmap))
                                    {
                                        myGraphic.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
                                        myGraphic.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
                                        myGraphic.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality;
                                        myGraphic.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;

                                        myGraphic.DrawImage(mySourceImage, 0, 0, sizeX, sizeY);
                                    }

                                    System.Drawing.Imaging.EncoderParameters myParams = new System.Drawing.Imaging.EncoderParameters(1);
                                    myParams.Param[0] = new System.Drawing.Imaging.EncoderParameter(System.Drawing.Imaging.Encoder.Quality, (long)80);

                                    myDestinationBitmap.Save(context.Response.OutputStream, System.Drawing.Imaging.ImageCodecInfo.GetImageEncoders()[1], myParams);
                                }
                            }
                            else
                            {
                                //no scaling required
                                context.Response.BufferOutput = true;
                                context.Response.BinaryWrite(StackBuffer);
                            }
                        }
                    }
                }

#if !DEBUG
                if (!myAlbum.ImageStackInError)
                {
                    context.Response.Cache.SetExpires(DateTime.Now.AddDays(14)); //cache for 2 weeks
                    context.Response.Cache.SetCacheability(HttpCacheability.Public);
                }
#endif
            }
            catch (Exception ex)
            {
                ImageError.RenderToBrowser(context.Response, ex.Message);
            }
        }

        #region IHttpHandler Members

        public bool IsReusable
        {
            get { return false; }
        }

        #endregion
    }
}
